/**
  *******************************************************************************************************
  * @file    tsi.h
  * @author  FMSH Application Team
  * @brief   Header file of FL Driver Library
  *******************************************************************************************************
  * @attention
  *
  * Copyright (c) [2021] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under Mulan PSL v2.
  * You can use this software according to the terms and conditions of the Mulan PSL v2.
  * You may obtain a copy of Mulan PSL v2 at:
  *          http://license.coscl.org.cn/MulanPSL2
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
  * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
  * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
  * See the Mulan PSL v2 for more details.
  *
  *******************************************************************************************************
  */
/* Prevent recursive inclusion ------------------------------------------------------------------------*/
#ifndef __TSI_LIB_H__
#define __TSI_LIB_H__

#ifdef __cplusplus
extern "C" {
#endif

/* Includes -------------------------------------------------------------------------------------------*/
#include "tsi_conf.h"
#include "tsi_objects.h"

/* Macros ---------------------------------------------------------------------------------------------*/
/**
  * @brief TSI Library version number
  */
#define __TSI_VERSION_MAIN          (0x01) /*!< [31:24] main version */
#define __TSI_VERSION_SUB1          (0x00) /*!< [23:16] sub1 version */
#define __TSI_VERSION_SUB2          (0x01) /*!< [15:0]  sub2 version */
#define __TSI_VERSION               ((__TSI_VERSION_MAIN  << 24)\
                                         |(__TSI_VERSION_SUB1 << 16)\
                                         |(__TSI_VERSION_SUB2))

/* Defines --------------------------------------------------------------------------------------------*/
typedef enum _TSI_ScanModeEnum
{
    TSI_SCAN_SINGLE = 0,
    TSI_SCAN_CONTINUOUS = 1,
} TSI_ScanModeType;

/* Exported Functions ---------------------------------------------------------------------------------*/
/** @defgroup TSI_CONTROL_FUNC  TSI Control Functions
  * @{
  */

TSI_RetCodeType TSI_Init(void);
TSI_RetCodeType TSI_Start(void);
TSI_RetCodeType TSI_Suspend(void);
TSI_RetCodeType TSI_Resume(void);
TSI_RetCodeType TSI_Reconfig(void);

/**
  * @}
  */

/** @defgroup TSI_ERROR_HANDLE_FUNC  TSI Error Handler Functions
  * @{
  */

void TSI_ErrorHandler(void* obj, uint32_t val, const char* msg);

/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif  /* __TSI_LIB_H__ */

/********************** (C) COPYRIGHT Fudan Microelectronics **** END OF FILE ***********************/
