/**
  *******************************************************************************************************
  * @file    tsi_def.h
  * @author  FMSH Application Team
  * @brief   Header file of FL Driver Library
  *******************************************************************************************************
  * @attention
  *
  * Copyright (c) [2021] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under Mulan PSL v2.
  * You can use this software according to the terms and conditions of the Mulan PSL v2.
  * You may obtain a copy of Mulan PSL v2 at:
  *          http://license.coscl.org.cn/MulanPSL2
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
  * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
  * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
  * See the Mulan PSL v2 for more details.
  *
  *******************************************************************************************************
  */
/* Prevent recursive inclusion ------------------------------------------------------------------------*/
#ifndef __TSI_LIB_DEF_H__
#define __TSI_LIB_DEF_H__

#ifdef __cplusplus
extern "C" {
#endif

/* Includes -------------------------------------------------------------------------------------------*/
#include <stdint.h>

#if defined(FM33FR0XX)
    #include "fm33fr0xx_fl_tsi.h"
#elif defined(FM33FT0XXA)
	#include "fm33ft0xxa_fl_tsi.h"	
#else
    #error Chip not supported!
#endif

/* Macros ---------------------------------------------------------------------------------------------*/
#if defined(__CC_ARM)   /* KEIL */
#define TSI_USED                    __attribute__((used))
#define TSI_SECTION(__NAME__)       __attribute__((section(__NAME__)))
#define TSI_ALIGN4                  __attribute__((aligned(4)))       
#elif defined(__IAR_SYSTEMS_ICC__)  /* IAR */
#define TSI_USED                    __root
#define TSI_SECTION(__NAME__)       @ __NAME__
#define TSI_ALIGN4                  _Pragma("data_alignment=4")                      
#endif

#define TSI_CONST_MODULE_SECTION        TSI_SECTION("tsi_lib_const.0_module")  
#define TSI_CONST_WIDGETS_SECTION       TSI_SECTION("tsi_lib_const.1_widgets")   
#define TSI_CONST_SENSORS_SECTION       TSI_SECTION("tsi_lib_const.2_sensors")
#define TSI_CONST_P_WIDGETS_SECTION     TSI_SECTION("tsi_lib_const.3_widgetPointers")
#define TSI_CONST_CH_SNS_MAP_SECTION    TSI_SECTION("tsi_lib_const.4_channelSensorMap")
#define TSI_CONST_META_WIDGETS_SECTION  TSI_SECTION("tsi_lib_const.5_metaWidgets")
#define TSI_CONST_META_SENSORS_SECTION  TSI_SECTION("tsi_lib_const.6_metaSensors")
#define TSI_CONST_PINS_SECTION          TSI_SECTION("tsi_lib_const.7_pins")

#define TSI_MODULE_SECTION              TSI_SECTION("tsi_lib.0_module")
#define TSI_WIDGETS_SECTION             TSI_SECTION("tsi_lib.1_widgets")   
#define TSI_SENSORS_SECTION             TSI_SECTION("tsi_lib.2_sensors")
#define TSI_FILTERS_SECTION             TSI_SECTION("tsi_lib.3_filters")
#define TSI_DEBOUNCES_SECTION           TSI_SECTION("tsi_lib.4_debounces")

#define TSI_WIDGET_ENABLE               (1U)
#define TSI_WIDGET_DISABLE              (0U)

#define TSI_SAMPLE_CLK_DIRECT           (0U)
#define TSI_SAMPLE_CLK_PRS              (1U)

#define TSI_CALIB_METHOD_NONE           (0U)
#define TSI_CALIB_METHOD_HARDWARE       (1U)

#define TSI_SENSOR_NORMAL_BASELINE      (0U)
#define TSI_SENSOR_IIR_BASELINE         (1U)

#define TSI_INACTIVE_SENSOR_FLOAT       (0U)
#define TSI_INACTIVE_SENSOR_GROUNDED    (1U)

/* Defines --------------------------------------------------------------------------------------------*/
typedef enum _TSI_RetCodeTypeEnum
{
    TSI_PASS            = 0,    /* No error */
    TSI_TIMEOUT         = 1,    /* Operation timeout */
    TSI_FSM_CORRUPTED   = 2,    /* TSI state machine corrupted */
    TSI_PARAM_ERR       = 3,    /* Wrong parameter passed */
    TSI_UNSUPPORTED     = 4,    /* Unsupported operation */
    TSI_OPERATION_ERR   = 5,    /* Operation failed */ 
} TSI_RetCodeType;

/* Exported Functions ---------------------------------------------------------------------------------*/

#ifdef __cplusplus
}
#endif

#endif  /* __TSI_LIB_DEF_H__ */

/********************** (C) COPYRIGHT Fudan Microelectronics **** END OF FILE ***********************/
