/**
  *******************************************************************************************************
  * @file    tsi_filter.h
  * @author  FMSH Application Team
  * @brief   Header file of FL Driver Library
  *******************************************************************************************************
  * @attention
  *
  * Copyright (c) [2021] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under Mulan PSL v2.
  * You can use this software according to the terms and conditions of the Mulan PSL v2.
  * You may obtain a copy of Mulan PSL v2 at:
  *          http://license.coscl.org.cn/MulanPSL2
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
  * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
  * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
  * See the Mulan PSL v2 for more details.
  *
  *******************************************************************************************************
  */
/* Prevent recursive inclusion ------------------------------------------------------------------------*/
#ifndef __TSI_LIB_FILTER_H__
#define __TSI_LIB_FILTER_H__

#ifdef __cplusplus
extern "C" {
#endif

/* Includes -------------------------------------------------------------------------------------------*/
#include "tsi_objects.h"

/* Macros ---------------------------------------------------------------------------------------------*/

/* Defines --------------------------------------------------------------------------------------------*/

/* Exported Functions ---------------------------------------------------------------------------------*/
/** @defgroup TSI_FLT_INIT_FUNC  TSI Filter Init Functions
  * @{
  */
  
void TSI_Filter_InitSensor(TSI_SensorTypeDef* sensor);
void TSI_Filter_Average4Init(uint16_t *buffer, uint16_t initVal);
void TSI_Filter_Median3Init(uint16_t *buffer, uint16_t initVal);
void TSI_Filter_JitterInit(uint16_t *buffer, uint16_t initVal);
void TSI_Filter_IIRInit(uint32_t *buffer, uint16_t initVal);
void TSI_Filter_ADVIIRInit(uint32_t *buffer, uint16_t initVal);

/**
  * @}
  */

/** @defgroup TSI_FLT_UPD_FUNC  TSI Filter Update Functions
  * @{
  */

void TSI_Filter_UpdateSensor(TSI_SensorTypeDef* sensor);
void TSI_Filter_Average4Update(uint16_t *buffer, uint16_t* input);
void TSI_Filter_Median3Update(uint16_t *buffer, uint16_t* input);
void TSI_Filter_JitterUpdate(uint16_t *buffer, uint16_t* input);
void TSI_Filter_IIRUpdate(uint32_t *buffer, uint8_t coef, uint16_t* input);
void TSI_Filter_ADVIIRUpdate(uint32_t *buffer, uint16_t* input, uint8_t filterMode);

/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif  /* __TSI_LIB_FILTER_H__ */

/********************** (C) COPYRIGHT Fudan Microelectronics **** END OF FILE ***********************/

