/**
  ****************************************************************************************************
  * @file    tsi_objects.c
  * @author  FMSH Application Team
  * @brief   Source file of FL Driver Library
  ****************************************************************************************************
  * @attention
  *
  * Copyright (c) [2021] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under Mulan PSL v2.
  * You can use this software according to the terms and conditions of the Mulan PSL v2.
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
  * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
  * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
  * See the Mulan PSL v2 for more details.
  *
  ****************************************************************************************************
  */
/* Includes ----------------------------------------------------------------------------------------*/
#include "tsi_objects.h"
#include <string.h>

/* Macros ---------------------------------------------------------------------------------------------*/

/* Defines --------------------------------------------------------------------------------------------*/
/* Objects ----------------------------------------------------------------------*/
TSI_ALIGN4 TSI_USED TSI_ModuleTypeDef TSI_Module TSI_MODULE_SECTION;
TSI_ALIGN4 TSI_USED TSI_WidgetListTypeDef TSI_WidgetList TSI_WIDGETS_SECTION;
TSI_ALIGN4 TSI_USED TSI_SensorListTypeDef TSI_SensorList TSI_SENSORS_SECTION;

/* Filter buffers ---------------------------------------------------------------*/
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterBtn1[TSI_BTN1_SENSORS_NUM];
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterBtn2[TSI_BTN2_SENSORS_NUM];
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterBtn3[TSI_BTN3_SENSORS_NUM];
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterBtn4[TSI_BTN4_SENSORS_NUM];
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterBtn5[TSI_BTN5_SENSORS_NUM];
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterBtn6[TSI_BTN6_SENSORS_NUM];
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterBtn7[TSI_BTN7_SENSORS_NUM];
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterBtn8[TSI_BTN8_SENSORS_NUM];
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterSlider[TSI_SLIDER_SENSORS_NUM];

/* Debounce buffers -------------------------------------------------------------*/
static TSI_USED uint8_t TSI_DebounceBtn1[TSI_BTN1_SENSORS_NUM];
static TSI_USED uint8_t TSI_DebounceBtn2[TSI_BTN2_SENSORS_NUM];
static TSI_USED uint8_t TSI_DebounceBtn3[TSI_BTN3_SENSORS_NUM];
static TSI_USED uint8_t TSI_DebounceBtn4[TSI_BTN4_SENSORS_NUM];
static TSI_USED uint8_t TSI_DebounceBtn5[TSI_BTN5_SENSORS_NUM];
static TSI_USED uint8_t TSI_DebounceBtn6[TSI_BTN6_SENSORS_NUM];
static TSI_USED uint8_t TSI_DebounceBtn7[TSI_BTN7_SENSORS_NUM];
static TSI_USED uint8_t TSI_DebounceBtn8[TSI_BTN8_SENSORS_NUM];
static TSI_USED uint8_t TSI_DebounceSlider[TSI_SLIDER_SENSORS_NUM];

/* Initial values ---------------------------------------------------------------*/
/* TSI Module */
TSI_ALIGN4 TSI_USED const TSI_ModuleTypeDef TSI_ModuleConstInit TSI_CONST_MODULE_SECTION = 
{
    2381172838,  /* firmwareId */
    0,  /* debugCmd */
    /* hardwareCfg */
    {
        0,  /* opClkSource */
        1,  /* opClkPrescaler */
        24000000,   /* opClkFreq */
    },
    /* scanCfg */
    {
        0,  /* scanPeriod */
        6,  /* sampleInterval */
        20,  /* sampleIdleTime */
    },
    TSI_LIB_RESET,  /* status */
};

/* TSI sensor list */
TSI_ALIGN4 TSI_USED const TSI_SensorListTypeDef TSI_SensorListConstInit TSI_CONST_SENSORS_SECTION = 
{
    {   
        {   /* Btn1_Sns0 */
            &TSI_MetaSensors[0],
            6U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
    {   
        {   /* Btn2_Sns0 */
            &TSI_MetaSensors[1],
            5U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
    {   
        {   /* Btn3_Sns0 */
            &TSI_MetaSensors[2],
            4U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
    {   
        {   /* Btn4_Sns0 */
            &TSI_MetaSensors[3],
            5U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
    {   
        {   /* Btn5_Sns0 */
            &TSI_MetaSensors[4],
            4U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
    {   
        {   /* Btn6_Sns0 */
            &TSI_MetaSensors[5],
            4U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
    {   
        {   /* Btn7_Sns0 */
            &TSI_MetaSensors[6],
            4U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
    {   
        {   /* Btn8_Sns0 */
            &TSI_MetaSensors[7],
            4U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
    {   
        {   /* Slider_Sns0 */
            &TSI_MetaSensors[8],
            4U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
        {   /* Slider_Sns1 */
            &TSI_MetaSensors[9],
            4U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
        {   /* Slider_Sns2 */
            &TSI_MetaSensors[10],
            5U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
        {   /* Slider_Sns3 */
            &TSI_MetaSensors[11],
            5U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
        {   /* Slider_Sns4 */
            &TSI_MetaSensors[12],
            5U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
        {   /* Slider_Sns5 */
            &TSI_MetaSensors[13],
            5U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
};

/* TSI widget list */
TSI_ALIGN4 TSI_USED const TSI_WidgetListTypeDef TSI_WidgetListConstInit TSI_CONST_WIDGETS_SECTION = 
{
    {   /* Btn1 */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[0],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            14U,	/* resolution */
            8U,	/* sampleClkDiv */
            6U,	/* idacMod */
            500U,	/* activeTh */
            20U,	/* activeHys */
            100U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            50U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
        0U,	/* buttonStatus */
    },
    {   /* Btn2 */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[1],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            14U,	/* resolution */
            8U,	/* sampleClkDiv */
            5U,	/* idacMod */
            500U,	/* activeTh */
            20U,	/* activeHys */
            100U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            50U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
        0U,	/* buttonStatus */
    },
    {   /* Btn3 */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[2],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            14U,	/* resolution */
            8U,	/* sampleClkDiv */
            5U,	/* idacMod */
            500U,	/* activeTh */
            20U,	/* activeHys */
            100U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            50U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
        0U,	/* buttonStatus */
    },
    {   /* Btn4 */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[3],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            14U,	/* resolution */
            8U,	/* sampleClkDiv */
            5U,	/* idacMod */
            500U,	/* activeTh */
            20U,	/* activeHys */
            100U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            50U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
        0U,	/* buttonStatus */
    },
    {   /* Btn5 */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[4],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            14U,	/* resolution */
            8U,	/* sampleClkDiv */
            5U,	/* idacMod */
            500U,	/* activeTh */
            20U,	/* activeHys */
            100U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            50U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
        0U,	/* buttonStatus */
    },
    {   /* Btn6 */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[5],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            14U,	/* resolution */
            8U,	/* sampleClkDiv */
            5U,	/* idacMod */
            500U,	/* activeTh */
            20U,	/* activeHys */
            100U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            50U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
        0U,	/* buttonStatus */
    },
    {   /* Btn7 */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[6],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            14U,	/* resolution */
            8U,	/* sampleClkDiv */
            4U,	/* idacMod */
            500U,	/* activeTh */
            20U,	/* activeHys */
            100U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            50U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
        0U,	/* buttonStatus */
    },
    {   /* Btn8 */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[7],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            14U,	/* resolution */
            8U,	/* sampleClkDiv */
            5U,	/* idacMod */
            500U,	/* activeTh */
            20U,	/* activeHys */
            100U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            50U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
        0U,	/* buttonStatus */
    },
    {   /* Slider */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[8],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            14U,	/* resolution */
            8U,	/* sampleClkDiv */
            6U,	/* idacMod */
            500U,	/* activeTh */
            20U,	/* activeHys */
            100U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            50U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
    },
};

/* Mapping ----------------------------------------------------------------------*/
TSI_USED TSI_SensorTypeDef* const TSI_ChannelSensorMap[TSI_TOTAL_CHANNEL_NUM] TSI_CONST_CH_SNS_MAP_SECTION =
{
    /* Channel 0 */
    &TSI_SensorList.slider[0],
    /* Channel 1 */
    &TSI_SensorList.slider[1],
    /* Channel 2 */
    &TSI_SensorList.slider[2],
    /* Channel 3 */
    &TSI_SensorList.slider[3],
    /* Channel 4 */
    &TSI_SensorList.slider[4],
    /* Channel 5 */
    &TSI_SensorList.slider[5],
    /* Channel 6 */
    &TSI_SensorList.btn5[0],
    /* Channel 7 */
    &TSI_SensorList.btn6[0],
    /* Channel 8 */
    &TSI_SensorList.btn7[0],
    /* Channel 9 */
    &TSI_SensorList.btn8[0],
    /* Channel 10 */
    &TSI_SensorList.btn4[0],
    /* Channel 11 */
    &TSI_SensorList.btn3[0],
    /* Channel 12 */
    &TSI_SensorList.btn2[0],
    /* Channel 13 */
    &TSI_SensorList.btn1[0],
};

/* TSI widget pointers */
TSI_USED TSI_WidgetTypeDef* const TSI_WidgetPointers[TSI_TOTAL_WIDGETS_NUM] TSI_CONST_P_WIDGETS_SECTION =
{
    (TSI_WidgetTypeDef*)&TSI_WidgetList.btn1,
    (TSI_WidgetTypeDef*)&TSI_WidgetList.btn2,
    (TSI_WidgetTypeDef*)&TSI_WidgetList.btn3,
    (TSI_WidgetTypeDef*)&TSI_WidgetList.btn4,
    (TSI_WidgetTypeDef*)&TSI_WidgetList.btn5,
    (TSI_WidgetTypeDef*)&TSI_WidgetList.btn6,
    (TSI_WidgetTypeDef*)&TSI_WidgetList.btn7,
    (TSI_WidgetTypeDef*)&TSI_WidgetList.btn8,
    (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,
};

/* TSI meta widgets */
TSI_USED TSI_MetaWidgetTypeDef const TSI_MetaWidgets[TSI_TOTAL_WIDGETS_NUM] TSI_CONST_META_WIDGETS_SECTION = 
{
    {   /* Btn1 */
        TSI_WIDGET_BUTTON,	/* type */
        TSI_SensorList.btn1,	/* sensors */
        TSI_BTN1_SENSORS_NUM,	/* sensorCnt */
    },
    {   /* Btn2 */
        TSI_WIDGET_BUTTON,	/* type */
        TSI_SensorList.btn2,	/* sensors */
        TSI_BTN2_SENSORS_NUM,	/* sensorCnt */
    },
    {   /* Btn3 */
        TSI_WIDGET_BUTTON,	/* type */
        TSI_SensorList.btn3,	/* sensors */
        TSI_BTN3_SENSORS_NUM,	/* sensorCnt */
    },
    {   /* Btn4 */
        TSI_WIDGET_BUTTON,	/* type */
        TSI_SensorList.btn4,	/* sensors */
        TSI_BTN4_SENSORS_NUM,	/* sensorCnt */
    },
    {   /* Btn5 */
        TSI_WIDGET_BUTTON,	/* type */
        TSI_SensorList.btn5,	/* sensors */
        TSI_BTN5_SENSORS_NUM,	/* sensorCnt */
    },
    {   /* Btn6 */
        TSI_WIDGET_BUTTON,	/* type */
        TSI_SensorList.btn6,	/* sensors */
        TSI_BTN6_SENSORS_NUM,	/* sensorCnt */
    },
    {   /* Btn7 */
        TSI_WIDGET_BUTTON,	/* type */
        TSI_SensorList.btn7,	/* sensors */
        TSI_BTN7_SENSORS_NUM,	/* sensorCnt */
    },
    {   /* Btn8 */
        TSI_WIDGET_BUTTON,	/* type */
        TSI_SensorList.btn8,	/* sensors */
        TSI_BTN8_SENSORS_NUM,	/* sensorCnt */
    },
    {   /* Slider */
        TSI_WIDGET_SLIDER,	/* type */
        TSI_SensorList.slider,	/* sensors */
        TSI_SLIDER_SENSORS_NUM,	/* sensorCnt */
    },
};

/* TSI meta sensors */
TSI_USED TSI_MetaSensorTypeDef const TSI_MetaSensors[TSI_TOTAL_SENSORS_NUM] TSI_CONST_META_SENSORS_SECTION = 
{
    {   /* Btn1_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.btn1,	/* parent */
        13,	/* channel */
        &TSI_FilterBtn1[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceBtn1[0],	/* debArray */
    },
    {   /* Btn2_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.btn2,	/* parent */
        12,	/* channel */
        &TSI_FilterBtn2[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceBtn2[0],	/* debArray */
    },
    {   /* Btn3_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.btn3,	/* parent */
        11,	/* channel */
        &TSI_FilterBtn3[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceBtn3[0],	/* debArray */
    },
    {   /* Btn4_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.btn4,	/* parent */
        10,	/* channel */
        &TSI_FilterBtn4[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceBtn4[0],	/* debArray */
    },
    {   /* Btn5_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.btn5,	/* parent */
        6,	/* channel */
        &TSI_FilterBtn5[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceBtn5[0],	/* debArray */
    },
    {   /* Btn6_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.btn6,	/* parent */
        7,	/* channel */
        &TSI_FilterBtn6[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceBtn6[0],	/* debArray */
    },
    {   /* Btn7_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.btn7,	/* parent */
        8,	/* channel */
        &TSI_FilterBtn7[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceBtn7[0],	/* debArray */
    },
    {   /* Btn8_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.btn8,	/* parent */
        9,	/* channel */
        &TSI_FilterBtn8[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceBtn8[0],	/* debArray */
    },
    {   /* Slider_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,	/* parent */
        0,	/* channel */
        &TSI_FilterSlider[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceSlider[0],	/* debArray */
    },
    {   /* Slider_Sns1 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,	/* parent */
        1,	/* channel */
        &TSI_FilterSlider[1],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceSlider[1],	/* debArray */
    },
    {   /* Slider_Sns2 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,	/* parent */
        2,	/* channel */
        &TSI_FilterSlider[2],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceSlider[2],	/* debArray */
    },
    {   /* Slider_Sns3 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,	/* parent */
        3,	/* channel */
        &TSI_FilterSlider[3],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceSlider[3],	/* debArray */
    },
    {   /* Slider_Sns4 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,	/* parent */
        4,	/* channel */
        &TSI_FilterSlider[4],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceSlider[4],	/* debArray */
    },
    {   /* Slider_Sns5 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,	/* parent */
        5,	/* channel */
        &TSI_FilterSlider[5],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceSlider[5],	/* debArray */
    },
};

/* TSI pins */
TSI_USED TSI_PinTypeDef const TSI_Pins[TSI_TOTAL_PINS_NUM] TSI_CONST_PINS_SECTION = 
{
    {   /* 0: Slider_Sns0 */
        0,
        4,
        0,
    },
    {   /* 1: Slider_Sns1 */
        1,
        4,
        1,
    },
    {   /* 2: Slider_Sns2 */
        2,
        4,
        2,
    },
    {   /* 3: Slider_Sns3 */
        3,
        4,
        3,
    },
    {   /* 4: Slider_Sns4 */
        4,
        4,
        4,
    },
    {   /* 5: Slider_Sns5 */
        3,
        2,
        5,
    },
    {   /* 6: Btn5_Sns0 */
        4,
        2,
        6,
    },
    {   /* 7: Btn6_Sns0 */
        7,
        3,
        7,
    },
    {   /* 8: Btn7_Sns0 */
        9,
        4,
        8,
    },
    {   /* 9: Btn8_Sns0 */
        10,
        4,
        9,
    },
    {   /* 10: Btn4_Sns0 */
        10,
        0,
        10,
    },
    {   /* 11: Btn3_Sns0 */
        11,
        0,
        11,
    },
    {   /* 12: Btn2_Sns0 */
        12,
        0,
        12,
    },
    {   /* 13: Btn1_Sns0 */
        11,
        3,
        13,
    },
};

/* Exported Functions ---------------------------------------------------------------------------------*/

/********************** (C) COPYRIGHT Fudan Microelectronics **** END OF FILE ***********************/

