/**
  ****************************************************************************************************
  * @file    tsi_port.h
  * @author  FMSH Application Team
  * @brief   Source file of FL Driver Library
  ****************************************************************************************************
  * @attention
  *
  * Copyright (c) [2021] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under Mulan PSL v2.
  * You can use this software according to the terms and conditions of the Mulan PSL v2.
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
  * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
  * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
  * See the Mulan PSL v2 for more details.
  *
  ****************************************************************************************************
  */
/* Includes ----------------------------------------------------------------------------------------*/
#include "tsi_port.h"
#include "tsi.h"
#include "fm33fr0xx_fl.h"

/* Macros ---------------------------------------------------------------------------------------------*/
#define GET_TSI_IO_PORT_ID(__PORT__)      (((GPIO_Type*)__PORT__) == GPIOA ? 0 : ( \
                                           ((GPIO_Type*)__PORT__) == GPIOB ? 1 : ( \
                                           ((GPIO_Type*)__PORT__) == GPIOC ? 2 : ( \
                                           ((GPIO_Type*)__PORT__) == GPIOD ? 3 : ( \
                                           ((GPIO_Type*)__PORT__) == GPIOE ? 4 : ( \
                                           ((GPIO_Type*)__PORT__) == GPIOF ? 5 : 0))))))

#define GET_TSI_IO_PIN_ID(__PIN__)        (((uint32_t)__PIN__) == FL_GPIO_PIN_0 ? 0 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_1 ? 1 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_2 ? 2 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_3 ? 3 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_4 ? 4 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_5 ? 5 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_6 ? 6 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_7 ? 7 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_8 ? 8 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_9 ? 9 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_10 ? 10 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_11 ? 11 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_12 ? 12 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_13 ? 13 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_14 ? 14 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_15 ? 15 : 0))))))))))))))))

/*
    GPIO Unique ID(16bit):
    - [15:10] Channel
    - [9:4]   Port (GPIOA, B...)
    - [3:0]   Pin (FL_GPIO_PIN_0 - FL_GPIO_PIN_15)
 */
#define GET_TSI_IO_UNIQUE_ID(__IO_INIT_STRUCT__)    ((uint16_t)((((FL_TSI_IOInitTypeDef*)(__IO_INIT_STRUCT__))->channel & 0x3FU) << 10U |    \
                                                               (GET_TSI_IO_PORT_ID(((FL_TSI_IOInitTypeDef*)(__IO_INIT_STRUCT__))->port) & 0x3FU) << 4U | \
                                                               (GET_TSI_IO_PIN_ID(((FL_TSI_IOInitTypeDef*)(__IO_INIT_STRUCT__))->pin) & 0xFU)))

#define IS_TSI_VALID_IO(__VALUE__)                  (((__VALUE__) == (uint16_t)(0U  << 10U | 4U << 4U |  0U))||     /* CH0  @ PE0  */   \
                                                     ((__VALUE__) == (uint16_t)(1U  << 10U | 4U << 4U |  1U))||     /* CH1  @ PE1  */   \
                                                     ((__VALUE__) == (uint16_t)(2U  << 10U | 4U << 4U |  2U))||     /* CH2  @ PE2  */   \
                                                     ((__VALUE__) == (uint16_t)(3U  << 10U | 4U << 4U |  3U))||     /* CH3  @ PE3  */   \
                                                     ((__VALUE__) == (uint16_t)(4U  << 10U | 4U << 4U |  4U))||     /* CH4  @ PE4  */   \
                                                     ((__VALUE__) == (uint16_t)(5U  << 10U | 2U << 4U |  3U))||     /* CH5  @ PC3  */   \
                                                     ((__VALUE__) == (uint16_t)(6U  << 10U | 2U << 4U |  4U))||     /* CH6  @ PC4  */   \
                                                     ((__VALUE__) == (uint16_t)(7U  << 10U | 3U << 4U |  7U))||     /* CH7  @ PD7  */   \
                                                     ((__VALUE__) == (uint16_t)(8U  << 10U | 4U << 4U |  9U))||     /* CH8  @ PE9  */   \
                                                     ((__VALUE__) == (uint16_t)(9U  << 10U | 4U << 4U | 10U))||     /* CH9  @ PE10 */   \
                                                     ((__VALUE__) == (uint16_t)(10U << 10U | 0U << 4U | 10U))||     /* CH10 @ PA10 */   \
                                                     ((__VALUE__) == (uint16_t)(11U << 10U | 0U << 4U | 11U))||     /* CH11 @ PA11 */   \
                                                     ((__VALUE__) == (uint16_t)(12U << 10U | 0U << 4U | 12U))||     /* CH12 @ PA12 */   \
                                                     ((__VALUE__) == (uint16_t)(13U << 10U | 3U << 4U | 11U)))      /* CH13 @ PD11 */

/* Defines --------------------------------------------------------------------------------------------*/
GPIO_Type *availablePorts[] =
{
    GPIOA, GPIOB, GPIOC, GPIOD, GPIOE,
    GPIOF, GPIOE,
};

/* Private Functions ---------------------------------------------------------------------------------*/
static void AutoSetScanPeriod(void)
{
    uint64_t scanTime = 0U;
    uint64_t opClkFreq = TSI_Module.hardwareCfg.opClkFreq /
                         TSI_Module.hardwareCfg.opClkPrescaler;
    int i;
    
    /* Calculate total scan time */
    for (i = 0; i < TSI_TOTAL_WIDGETS_NUM; i++)
    {
        const TSI_WidgetTypeDef* widget = TSI_WidgetPointers[i];
        
        /* 
            Scan time for one widget = 1/f_opclk * (2^N) * n_sns 
                                       + t_smpl + t_intv
         */
        scanTime += ((uint64_t)1000000U << widget->resolution) 
                    * widget->meta->sensorCnt / opClkFreq;
        scanTime += (TSI_Module.scanCfg.sampleIdleTime + 1) << 5U;
        scanTime += (TSI_Module.scanCfg.sampleInterval + 1) << 5U;
    }
    
    /* Select an appropriate scan period */
    FL_TSI_WriteScanPeriod(TSI, (uint32_t)((scanTime / 20000U) & 0xFFFFFFFFUL));
}

/* Exported Functions ---------------------------------------------------------------------------------*/
TSI_RetCodeType TSI_LL_Init(void)
{
    uint32_t priority;
    int i;
    
    /* Clock init -------------------------------------------*/
    FL_CMU_EnableGroup2BusClock(FL_CMU_GROUP2_BUSCLK_TSI);
    if(FL_CMU_GetAPBClockFreq() != TSI_Module.hardwareCfg.opClkFreq)
    {
        return TSI_OPERATION_ERR;
    }

    /* Pin init ---------------------------------------------*/
    for (i = 0; i < TSI_TOTAL_PINS_NUM; i++)
    {
        FL_GPIO_InitTypeDef gpio = {0};
        uint32_t portId = ((TSI_Pins[i].pinUniqueId & 0x3F0U) >> 4U);
        uint32_t pin = ((uint32_t)1U << (TSI_Pins[i].pinUniqueId & 0xFU));
        GPIO_Type *port;

        assert_param(IS_TSI_VALID_IO(TSI_Pins[i].pinUniqueId));
        assert_param(portId < (sizeof(availablePorts) / sizeof(GPIO_Type *)));
        port = availablePorts[portId];

        FL_GPIO_StructInit(&gpio);
        gpio.pin = pin;
        gpio.mode = (uint32_t)FL_GPIO_MODE_ANALOG;
        (void)FL_GPIO_Init(port, &gpio);
    }

    /* Interrupt init ---------------------------------------*/
    priority = TSI_INTERRUPT_PRIORITY;
    if(priority >= 3)
    {
        priority = 3;
    }
    NVIC_DisableIRQ(TSI_IRQn);
    NVIC_ClearPendingIRQ(TSI_IRQn);
    NVIC_SetPriority(TSI_IRQn, priority);
    NVIC_EnableIRQ(TSI_IRQn);

    /* Config peripheral mode -------------------------------*/
    FL_TSI_SetStopMode(TSI, FL_TSI_STOP_MODE_IMMEDIATELY);

    return TSI_PASS;
}

TSI_RetCodeType TSI_LL_Reset(void)
{
    FL_TSI_ModuleReset();

    return TSI_PASS;
}

TSI_RetCodeType TSI_LL_Start(void)
{
    /* Enable EOC & EOS interrupt */
    FL_TSI_ClearFlag_EndOfChannel(TSI);
    FL_TSI_EnableIT_EndOfChannel(TSI);
    FL_TSI_ClearFlag_EndOfSequence(TSI);
    FL_TSI_EnableIT_EndOfSequence(TSI);

    /* Start scanning */
    FL_TSI_Enable(TSI);
    FL_TSI_EnableScan(TSI);
    
    return TSI_PASS;
}

TSI_RetCodeType TSI_LL_Stop(void)
{
    /* Stop scanning immediately */
    FL_TSI_SetStopMode(TSI, FL_TSI_STOP_MODE_IMMEDIATELY);
    FL_TSI_Disable(TSI);

    /* Disable EOC & EOS interrupt */
    FL_TSI_DisableIT_EndOfChannel(TSI);
    FL_TSI_ClearFlag_EndOfChannel(TSI);
    FL_TSI_DisableIT_EndOfSequence(TSI);
    FL_TSI_ClearFlag_EndOfSequence(TSI);

    return TSI_PASS;
}

void TSI_LL_SetSampleClockSource(uint32_t source)
{
    if(source == FL_TSI_SAMPLE_CLK_SOURCE_FIXED_CLK)
    {
        FL_TSI_SetSampleClockSource(TSI, FL_TSI_SAMPLE_CLK_SOURCE_FIXED_CLK);
    }
    else if (source == FL_TSI_SAMPLE_CLK_SOURCE_PRS_CLK)
    {
        FL_TSI_SetSampleClockSource(TSI, FL_TSI_SAMPLE_CLK_SOURCE_PRS_CLK);
    }
    else
    {
        /* 不应来到此处 */
    }
}

void TSI_LL_EnableShield(void)
{
    FL_TSI_EnableShield(TSI);
}

void TSI_LL_DisableShield(void)
{
    FL_TSI_DisableShield(TSI);
}

void TSI_LL_SetInactiveSensorConnection(uint32_t connect)
{
    if(connect == TSI_INACTIVE_SENSOR_FLOAT)
    {
        FL_TSI_SetInactiveSensorConnection(TSI, FL_TSI_SCAN_MODE_CONTINUOUS);
    }
    else if(connect == TSI_INACTIVE_SENSOR_GROUNDED)
    {
        FL_TSI_SetInactiveSensorConnection(TSI, FL_TSI_SCAN_MODE_SINGLE);
    }
    else
    {
        /* Do nothing */
    }
}

void TSI_LL_SetScanMode(uint32_t scanMode)
{
    if(scanMode == TSI_SCAN_MODE_CONTINUOUS)
    {
        FL_TSI_SetScanMode(TSI, FL_TSI_SCAN_MODE_CONTINUOUS);
    }
    else if(scanMode == TSI_SCAN_MODE_SINGLE)
    {
        FL_TSI_SetScanMode(TSI, FL_TSI_SCAN_MODE_SINGLE);
    }
    else
    {
        /* Do nothing */
    }
}

uint32_t TSI_LL_GetScanMode(void)
{
    return (FL_TSI_GetScanMode(TSI) == TSI_SCAN_MODE_CONTINUOUS) ?
        TSI_SCAN_MODE_CONTINUOUS : TSI_SCAN_MODE_SINGLE;
}

void TSI_LL_SetScanTiming(TSI_ScanTypeDef scanCfg)
{
    AutoSetScanPeriod();
    if(FL_TSI_ReadScanPeriod(TSI) < scanCfg.scanPeriod)
    {
        FL_TSI_WriteScanPeriod(TSI, scanCfg.scanPeriod);
    }
    FL_TSI_WriteScanIntervalTime(TSI, scanCfg.sampleInterval);
    FL_TSI_WriteScanIdleTime(TSI, scanCfg.sampleIdleTime);
}

TSI_RetCodeType TSI_LL_ConfigSensor(TSI_SensorTypeDef* sensor)
{
    uint32_t tmp;
    const TSI_MetaSensorTypeDef* metaSensor = sensor->meta;
    TSI_WidgetTypeDef* widget = metaSensor->parent;

    /* Apply sensor channel configurations ----------------------------------*/
    /* Set resolution */
    if(widget->resolution < 8U || widget->resolution > 16U)
    {
        return TSI_PARAM_ERR;
    }
    FL_TSI_SetResolution(TSI, widget->resolution - 8U, metaSensor->channel);

    /* Set sample clock prescaler */
    switch(widget->sampleClkDiv)
    {
        case 1: tmp = FL_TSI_SAMPLE_PSC_DIV1; break;
        case 2: tmp = FL_TSI_SAMPLE_PSC_DIV2; break;
        case 4: tmp = FL_TSI_SAMPLE_PSC_DIV4; break;
        case 8: tmp = FL_TSI_SAMPLE_PSC_DIV8; break;
        case 16: tmp = FL_TSI_SAMPLE_PSC_DIV16; break;
        case 32: tmp = FL_TSI_SAMPLE_PSC_DIV32; break;
        
        default:
            return TSI_PARAM_ERR;
    }
    FL_TSI_SetSampleClockPrescaler(TSI, tmp, metaSensor->channel);
    
    /* Set modulate IDAC current */
    tmp = widget->idacMod;
    
    if(tmp > 127U || tmp == 0U)
    {
        return TSI_PARAM_ERR;
    }
    FL_TSI_WriteMasterIDACCurrent(TSI, tmp, metaSensor->channel);

    /* Set compensation IDAC current */
#if (TSI_USE_COMP_IDAC)   
    tmp = sensor->idacComp;
    
    if(tmp > 127U)
    {
        return TSI_PARAM_ERR;
    }
    if(tmp != 0U)
    {
        FL_TSI_WriteCompensateIDACCurrent(TSI, tmp, metaSensor->channel);
        
        /* Set to dual IDAC mode when comensation IDAC current is not 0 */
        FL_TSI_SetIDACMode(TSI, FL_TSI_IDAC_MODE_DUAL, metaSensor->channel);
    }
    else    
    {
        /* Set to single IDAC mode when comensation IDAC current is 0 */
        FL_TSI_SetIDACMode(TSI, FL_TSI_IDAC_MODE_SINGLE, metaSensor->channel);
    }
#else
    /* Set to single IDAC mode */
    FL_TSI_SetIDACMode(TSI, FL_TSI_IDAC_MODE_SINGLE, metaSensor->channel);
#endif
    
    return TSI_PASS;
}

void TSI_LL_EnableSensor(TSI_SensorTypeDef* sensor)
{
    FL_TSI_SetNormSensorChannel(TSI, FL_TSI_GetNormSensorChannel(TSI) | (1 << sensor->meta->channel));
}

void TSI_LL_DisableSensor(TSI_SensorTypeDef* sensor)
{
    FL_TSI_SetNormSensorChannel(TSI, FL_TSI_GetNormSensorChannel(TSI) & ~(1 << sensor->meta->channel));
}

void TSI_IRQHandler(void)
{
    /* RawCount Update IRQ */
    if(FL_TSI_IsEnabledIT_EndOfChannel(TSI) && 
       FL_TSI_IsActiveFlag_EndOfChannel(TSI))
    {
        FL_TSI_ClearFlag_EndOfChannel(TSI);
        TSI_CHANNEL_SENSOR(FL_TSI_GetRawCountChannel(TSI))->rawCount = 
            FL_TSI_ReadRawCount(TSI); 
    }
    
    /* Sequence cplt IRQ */
    if(FL_TSI_IsEnabledIT_EndOfSequence(TSI) && 
       FL_TSI_IsActiveFlag_EndOfSequence(TSI))
    {
        FL_TSI_ClearFlag_EndOfSequence(TSI);
        
        /* Mark the end of sequence */
        TSI_SETSTAT_SCAN_CPLT();
    }
}

/********************** (C) COPYRIGHT Fudan Microelectronics **** END OF FILE ***********************/

