/**
  *******************************************************************************************************
  * @file    tsi_port.h
  * @author  FMSH Application Team
  * @brief   Header file of FL Driver Library
  *******************************************************************************************************
  * @attention
  *
  * Copyright (c) [2021] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under Mulan PSL v2.
  * You can use this software according to the terms and conditions of the Mulan PSL v2.
  * You may obtain a copy of Mulan PSL v2 at:
  *          http://license.coscl.org.cn/MulanPSL2
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
  * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
  * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
  * See the Mulan PSL v2 for more details.
  *
  *******************************************************************************************************
  */
/* Prevent recursive inclusion ------------------------------------------------------------------------*/
#ifndef __TSI_LIB_PORT_H__
#define __TSI_LIB_PORT_H__

#ifdef __cplusplus
extern "C" {
#endif

/* Includes -------------------------------------------------------------------------------------------*/
#include "fm33fr0xx_fl.h"
#include "tsi_def.h"
#include "tsi_objects.h"

/* Macros ---------------------------------------------------------------------------------------------*/

/* Defines --------------------------------------------------------------------------------------------*/

/* Exported Functions ---------------------------------------------------------------------------------*/
TSI_RetCodeType TSI_LL_Init(void);
TSI_RetCodeType TSI_LL_Reset(void);
TSI_RetCodeType TSI_LL_Start(void);
TSI_RetCodeType TSI_LL_Stop(void);

void TSI_LL_SetSampleClockSource(uint32_t source);

void TSI_LL_EnableShield(void);
void TSI_LL_DisableShield(void);

void TSI_LL_SetInactiveSensorConnection(uint32_t connect);
void TSI_LL_SetScanMode(uint32_t scanMode);
uint32_t TSI_LL_GetScanMode(void);
void TSI_LL_SetScanTiming(TSI_ScanTypeDef scanCfg);

TSI_RetCodeType TSI_LL_ConfigSensor(TSI_SensorTypeDef* sensor);

void TSI_LL_EnableSensor(TSI_SensorTypeDef* sensor);
void TSI_LL_DisableSensor(TSI_SensorTypeDef* sensor);

#ifdef __cplusplus
}
#endif

#endif  /* __TSI_LIB_PORT_H__ */

/********************** (C) COPYRIGHT Fudan Microelectronics **** END OF FILE ***********************/
