#ifndef __TSI_LIB_DEBUG_H__
#define __TSI_LIB_DEBUG_H__

#include <stdint.h>

/**
 *  @brief TSI debug protocol bit-field definition
 */
#define HEADER_DIR(__BUFFER__)                      ((uint8_t)((__BUFFER__[0] & 0x01U) >> 0U))
#define HEADER_ID(__BUFFER__)                       ((uint8_t)((__BUFFER__[0] & 0xFEU) >> 1U))
#define HEADER_PARITY(__BUFFER__)                   ((uint8_t)((__BUFFER__[1] & 0x0FU) >> 0U))
#define HEADER_ACCESS_SIZE(__BUFFER__)              ((uint8_t)((__BUFFER__[1] & 0x30U) >> 4U))
#define HEADER_ACCESS_WIDTH(__BUFFER__)             ((uint8_t)((__BUFFER__[1] & 0xC0U) >> 6U))
#define HEADER_OFFSET(__BUFFER__)                   ((uint16_t)(((uint16_t)__BUFFER__[3]) << 8U | __BUFFER__[2]))
#define BULK_HEADER_SIZE(__BUFFER__)                ((uint16_t)(((uint16_t)__BUFFER__[3]) << 8U | __BUFFER__[2]))

#define ACK_ERRCODE(__BUFFER__)                     ((uint8_t)((__BUFFER__ & 0x1FU) >> 0U))
#define ACK_STATUS(__BUFFER__)                      ((uint8_t)((__BUFFER__ & 0xE0U) >> 5U))
#define WRITE_ACK_ERRCODE(__BUFFER__, __DATA__)     (__BUFFER__ = ((__BUFFER__ & (~0x1FU)) | ((__DATA__ & 0x1FU) << 0U)))    
#define WRITE_ACK_STATUS(__BUFFER__, __DATA__)      (__BUFFER__ = ((__BUFFER__ & (~0xE0U)) | ((__DATA__ & 0x07U) << 5U)))     


/**
 *  @brief TSI debug protocol bulk-read buffer size
 */
#define TRACE_VAR_MAX_NUM                       64
#define TRACE_BUFFER_MAX_SIZE                   512

/**
 *  @brief TSI debug protocol status definition
 */
typedef enum{
    TSI_DBG_RDY = 0x0,
    TSI_DBG_ERR = 0x1,
    TSI_DBG_EOT = 0x2,
    TSI_DBG_RHS = 0x3,
    TSI_DBG_EHS = 0x4,
}TSI_Debug_Status;

/**
 *  @brief TSI debug protocol error code definition
 */
typedef enum{
    TSI_DBG_ERR_NONE = 0x0,
    TSI_DBG_ERR_HEAD_PARITY = 0x1,
    TSI_DBG_ERR_HEAD_FORMAT = 0x2,
    TSI_DBG_ERR_DATA_PARITY = 0x3,
    TSI_DBG_ERR_UNKNOWN = 0x1F,
}TSI_Debug_ErrorCode;

/**
 *  @brief TSI debug protocol trace var definition
 */
typedef struct _TSI_Debug_TraceVar_Struct
{
    uint8_t valid;
    uint8_t size;
    uint16_t addr;
} TSI_Debug_TraceVarTypeDef;

// Init debug protocol
void TSI_Debug_Init(void);

// Start debug protocol
void TSI_Debug_Start(void);

// Restart debug protocol
void TSI_Debug_Restart(void);

// Stop debug protocol
void TSI_Debug_Stop(void);

// Deinit debug protocol
void TSI_Debug_Deinit(void);

// Handle debug protocol event
void TSI_Debug_Handler(void);

// Update trace buffer, shall be called after the widget has been updated
void TSI_Debug_UpdateTrace(void);


#endif
