/**
  ****************************************************************************************************
  * @file    tsi_debug_io.c
  * @author  FMSH Application Team
  * @brief   Source file of FL Driver Library
  ****************************************************************************************************
  * @attention
  *
  * Copyright (c) [2021] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under Mulan PSL v2.
  * You can use this software according to the terms and conditions of the Mulan PSL v2.
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
  * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
  * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
  * See the Mulan PSL v2 for more details.
  *
  ****************************************************************************************************
  */
#include "tsi_debug_io.h"
#include "fm33fr0xx_fl.h"

/* Private includes ---------------------------------------------------------*/


/* USER PRIVATE INCLUDES BEGIN */

/* USER PRIVATE INCLUDES END */

/* Private variables --------------------------------------------------------*/
static uint16_t txCnt = 0, rxCnt = 0;
static uint8_t *txBuf = NULL, *rxBuf = NULL;

/* USER PRIVATE VARIABLES BEGIN */

/* USER PRIVATE VARIABLES END */

/* Private functions --------------------------------------------------------*/
void MUX7_IRQHandler(void)
{
    /* Ensure that MUX7 is selected as UART4 */
    if(FL_INTMUX_GetMUX7SEL() == FL_INTMUX_MUX7SEL_UART4)
    {
        /* Transmit */
        // Transmit buffer full
        if(FL_UART_IsEnabledIT_TXBuffEmpty(UART4) && FL_UART_IsActiveFlag_TXBuffEmpty(UART4))
        {
            FL_UART_WriteTXBuff(UART4, *txBuf++);
            
            txCnt--;  
            if(txCnt == 0)
            {
                FL_UART_DisableIT_TXBuffEmpty(UART4);
                FL_UART_EnableIT_TXShiftBuffEmpty(UART4);
            }
        }
        
        // Transmit complete
        if(FL_UART_IsEnabledIT_TXShiftBuffEmpty(UART4) && FL_UART_IsActiveFlag_TXShiftBuffEmpty(UART4))
        {
            FL_UART_DisableIT_TXShiftBuffEmpty(UART4);
            TSI_DebugIO.emitSignal(TSI_DebugIO_SigTxComplete);
        }
        
        /* Receive */
        // Receive buffer full
        if(FL_UART_IsEnabledIT_RXBuffFull(UART4) && FL_UART_IsActiveFlag_RXBuffFull(UART4))
        {
            *rxBuf++ = FL_UART_ReadRXBuff(UART4);
            
            rxCnt--;
            if(rxCnt == 0)
            {
                FL_UART_DisableIT_RXBuffFull(UART4);
                TSI_DebugIO.emitSignal(TSI_DebugIO_SigRxComplete);
            }
        }
    }
}

/* USER PRIVATE FUNCTIONS BEGIN */

/* USER PRIVATE FUNCTIONS END */

/* Implementations ----------------------------------------------------------*/
/**
  * @brief  Debug IO init function implementation
  * @note   Users should implement this function according to their hardware configuration.
  * @param  None
  * @retval None
  */
void TSI_DebugIO_Init(void)
{
    /* USER PRE DEBUG IO INIT BEGIN */

    /* USER PRE DEBUG IO INIT END */

    FL_UART_InitTypeDef uart = {0};
    FL_GPIO_InitTypeDef gpio = {0};
    
    FL_GPIO_StructInit(&gpio);
    gpio.pin = FL_GPIO_PIN_2 | FL_GPIO_PIN_3;
    gpio.mode = FL_GPIO_MODE_DIGITAL;
    gpio.pull = FL_GPIO_BOTH_DISABLE;
    gpio.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    gpio.remapPin = FL_GPIO_PINREMAP_FUNCTON1;
    FL_GPIO_Init(GPIOB, &gpio);
    
    uart.baudRate = 115200;
    uart.dataWidth = FL_UART_DATA_WIDTH_8B;
    uart.parity = FL_UART_PARITY_NONE;
    uart.stopBits = FL_UART_STOP_BIT_WIDTH_1B;
    uart.transferDirection = FL_UART_DIRECTION_TX_RX;
    FL_UART_Init(UART4, &uart);
    
    NVIC_DisableIRQ(MUX7_IRQn);
    FL_INTMUX_SetMUX7SEL(FL_INTMUX_MUX7SEL_UART4);
    NVIC_SetPriority(MUX7_IRQn, 2);
    NVIC_EnableIRQ(MUX7_IRQn);

    /* USER DEBUG IO INIT BEGIN */
    
    /* USER DEBUG IO INIT END */
}

/**
  * @brief  Debug IO deinit function implementation
  * @note   Users should implement this function according to their hardware configuration.
  * @param  None
  * @retval None
  */
void TSI_DebugIO_Deinit(void)
{
    /* USER PRE DEBUG IO DE-INIT BEGIN */

    /* USER PRE DEBUG IO DE-INIT END */

    FL_GPIO_DeInit(GPIOD, FL_GPIO_PIN_11 | FL_GPIO_PIN_12);
    FL_UART_DeInit(UART4);

    /* USER DEBUG IO DE-INIT BEGIN */

    /* USER DEBUG IO DE-INIT END */
}

/**
  * @brief  Debug IO clear function implementation
  * @note   Users should implement this function according to their hardware configuration.
  * @param  None
  * @retval None
  */
void TSI_DebugIO_Clear(void)
{
    /* USER PRE DEBUG IO CLEAR BEGIN */

    /* USER PRE DEBUG IO CLEAR END */

    // Read the receive buffer to clear the receive complete flag
    (void)FL_UART_ReadRXBuff(UART4);

    /* USER DEBUG IO CLEAR BEGIN */

    /* USER DEBUG IO CLEAR END */
}

/**
  * @brief  Debug IO transmit function implementation
  * @note   Users should implement this function according to their hardware configuration.
  * @param  pBuf    pointer to the transmit data buffer  
  * @param  length  data length
  * @retval None
  */
void TSI_DebugIO_Transmit(uint8_t *pBuf, uint16_t length)
{
    /* USER PRE DEBUG IO TRANSMIT BEGIN */
    
    /* USER PRE DEBUG IO TRANSMIT END */

    txBuf = pBuf;
    txCnt = length;
    
    FL_UART_EnableIT_TXBuffEmpty(UART4);

    /* USER DEBUG IO TRANSMIT BEGIN */

    /* USER DEBUG IO TRANSMIT END */
}

/**
  * @brief  Debug IO receive function implementation
  * @note   Users should implement this function according to their hardware configuration.
  * @param  pBuf    pointer to the receive data buffer  
  * @param  length  data length
  * @retval None
  */
void TSI_DebugIO_Receive(uint8_t *pBuf, uint16_t length)
{
    /* USER PRE DEBUG IO RECEIVE BEGIN */

    /* USER PRE DEBUG IO RECEIVE END */

    rxBuf = pBuf;
    rxCnt = length;
    
    FL_UART_EnableIT_RXBuffFull(UART4);

    /* USER DEBUG IO RECEIVE BEGIN */

    /* USER DEBUG IO RECEIVE END */
}

/**
  * @brief Debug IO instance 
  */
TSI_DebugIOTypeDef TSI_DebugIO = {
    TSI_DebugIO_Init,        // IO Init
    TSI_DebugIO_Deinit,      // IO Deinit
    TSI_DebugIO_Clear,       // IO Clear
    TSI_DebugIO_Transmit,    // IO Transmit
    TSI_DebugIO_Receive,     // IO Receive
    NULL,                    // Reserved for library to use, should be kepted as NULL
};

/********************** (C) COPYRIGHT Fudan Microelectronics **** END OF FILE ***********************/

