/**
  *****************************************************************************
  * @file    main.c
  * @author  FMSH Application Team
  * @brief   Main program
  *****************************************************************************
  * @attention
  *
  * Copyright (c) [2021] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under Mulan PSL v2.
  * You can use this software according to the terms and conditions of the Mulan PSL v2.
  * You may obtain a copy of Mulan PSL v2 at:
  *          http://license.coscl.org.cn/MulanPSL2
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
  * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
  * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
  * See the Mulan PSL v2 for more details.
  *
  *****************************************************************************
  */

/* Private includes ---------------------------------------------------------*/
/* Library includes */
#include "tsi.h"
#include "tsi_widget.h"

/* Library debug interface includes */
#include "tsi_debug.h"

/* FL driver library include */
#include "fm33fr0xx_fl.h"

/* USER PRIVATE INCLUDES BEGIN */
#include "led.h"
/* USER PRIVATE INCLUDES END */

/* Private variables --------------------------------------------------------*/
/* USER PRIVATE VARIABLES BEGIN */
/* h = [0,360], s = [0,1], v=[0,1] */
float H=0, S=1, V=0.5f;  
static uint8_t btnPressed = 0;
static uint8_t dispSliderVal = 0, dispSliderValCnt = 0;

/* Digit LED disp table 0-9, - */ 
static const uint8_t digiDispTable[11] = 
{
    0x3F,   /* 0 */
    0x06,   /* 1 */
    0x5B,   /* 2 */
    0x4F,   /* 3 */
    0x66,   /* 4 */
    0x6D,   /* 5 */
    0x7D,   /* 6 */
    0x07,   /* 7 */
    0x7F,   /* 8 */
    0x6F,   /* 9 */
    0x40,   /* - */
};

/* Digit LED disp vars */
static uint8_t digits[2] = {0, 0};
static uint8_t currDigit = 0;

#define DIG1_PWR_ON()       FL_GPIO_ResetOutputPin(GPIOD, FL_GPIO_PIN_14)
#define DIG1_PWR_OFF()      FL_GPIO_SetOutputPin(GPIOD, FL_GPIO_PIN_14)
#define DIG2_PWR_ON()       FL_GPIO_ResetOutputPin(GPIOC, FL_GPIO_PIN_15)
#define DIG2_PWR_OFF()      FL_GPIO_SetOutputPin(GPIOC, FL_GPIO_PIN_15)

#define SEG_A_ON()          FL_GPIO_ResetOutputPin(GPIOD, FL_GPIO_PIN_1)
#define SEG_A_OFF()         FL_GPIO_SetOutputPin(GPIOD, FL_GPIO_PIN_1)
#define SEG_B_ON()          FL_GPIO_ResetOutputPin(GPIOD, FL_GPIO_PIN_5)
#define SEG_B_OFF()         FL_GPIO_SetOutputPin(GPIOD, FL_GPIO_PIN_5)
#define SEG_C_ON()          FL_GPIO_ResetOutputPin(GPIOD, FL_GPIO_PIN_0)
#define SEG_C_OFF()         FL_GPIO_SetOutputPin(GPIOD, FL_GPIO_PIN_0)
#define SEG_D_ON()          FL_GPIO_ResetOutputPin(GPIOD, FL_GPIO_PIN_8)
#define SEG_D_OFF()         FL_GPIO_SetOutputPin(GPIOD, FL_GPIO_PIN_8)
#define SEG_E_ON()          FL_GPIO_ResetOutputPin(GPIOD, FL_GPIO_PIN_12)
#define SEG_E_OFF()         FL_GPIO_SetOutputPin(GPIOD, FL_GPIO_PIN_12)
#define SEG_F_ON()          FL_GPIO_ResetOutputPin(GPIOD, FL_GPIO_PIN_13)
#define SEG_F_OFF()         FL_GPIO_SetOutputPin(GPIOD, FL_GPIO_PIN_13)
#define SEG_G_ON()          FL_GPIO_ResetOutputPin(GPIOC, FL_GPIO_PIN_12)
#define SEG_G_OFF()         FL_GPIO_SetOutputPin(GPIOC, FL_GPIO_PIN_12)
#define SEG_DP_ON()         FL_GPIO_ResetOutputPin(GPIOD, FL_GPIO_PIN_6)
#define SEG_DP_OFF()        FL_GPIO_SetOutputPin(GPIOD, FL_GPIO_PIN_6)

/* USER PRIVATE VARIABLES END */

/* Private function prototypes ----------------------------------------------*/ 
static void SystemClockInit(void);

/* USER PRIVATE FUNCTION PROTOTYPES BEGIN */
static void DemoLed_ShowHSV(float h, float s, float v);
static void DigitLed_ShowSliderNum(uint8_t num);
static void DigitLed_ShowBtnNum(uint8_t num);
/* USER PRIVATE FUNCTION PROTOTYPES END */

/* Main ---------------------------------------------------------------------*/
int main(void)
{
    /* USER CODES AREA 1 BEGIN */
    FL_GPIO_InitTypeDef demo_led_io = {0};
    FL_BSTIM16_InitTypeDef demo_btim = {0};
    /* USER CODES AREA 1 END */

    /* Init FL driver library */
    FL_Init();

    /* Init system clock */
    SystemClockInit();

    /* USER CODES AREA 2 BEGIN */
    LED_Init();
    
    /* RGB LED */
    demo_led_io.pin        = FL_GPIO_PIN_6 | FL_GPIO_PIN_7 | FL_GPIO_PIN_8;
	demo_led_io.mode       = FL_GPIO_MODE_DIGITAL;
	demo_led_io.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    demo_led_io.pull       = FL_GPIO_BOTH_DISABLE;
	demo_led_io.remapPin   = FL_GPIO_PINREMAP_FUNCTON3;	
    demo_led_io.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
	FL_GPIO_Init(GPIOB, &demo_led_io);
    
    FL_CMU_EnableGroup4BusClock(FL_CMU_GROUP4_BUSCLK_TAU0);
    FL_CMU_EnableGroup4BusClock(FL_CMU_GROUP4_BUSCLK_TAU1);

    FL_TAU_SetOperationMode(TAU11, FL_TAU_OPERATION_MODE_OC);
    FL_TAU_WritePrescaler(TAU11, 24 - 1);
    FL_TAU_WriteAutoReload(TAU11, 100 - 1);
    FL_TAU_WriteCompare(TAU11, 1 - 1);
    FL_TAU_OC_SetPolarity(TAU11, FL_TAU_OC_POLARITY_NORMAL);
    FL_TAU_EnableOutput(TAU11);
    
    FL_TAU_SetOperationMode(TAU02, FL_TAU_OPERATION_MODE_OC);
    FL_TAU_WritePrescaler(TAU02, 24 - 1);
    FL_TAU_WriteAutoReload(TAU02, 100 - 1);
    FL_TAU_WriteCompare(TAU02, 1 - 1);
    FL_TAU_OC_SetPolarity(TAU02, FL_TAU_OC_POLARITY_NORMAL);
    FL_TAU_EnableOutput(TAU02);
    
    FL_TAU_SetOperationMode(TAU13, FL_TAU_OPERATION_MODE_OC);
    FL_TAU_WritePrescaler(TAU13, 24 - 1);
    FL_TAU_WriteAutoReload(TAU13, 100 - 1);
    FL_TAU_WriteCompare(TAU13, 1 - 1);
    FL_TAU_OC_SetPolarity(TAU13, FL_TAU_OC_POLARITY_NORMAL);
    FL_TAU_EnableOutput(TAU13);
    
    FL_TAU_Enable(TAU11);
    FL_TAU_Enable(TAU02);
    FL_TAU_Enable(TAU13);
    
    /* Digit LED */
    demo_led_io.pin        = FL_GPIO_PIN_12 | FL_GPIO_PIN_15;
	demo_led_io.mode       = FL_GPIO_MODE_OUTPUT;
    FL_GPIO_Init(GPIOC, &demo_led_io);
    
    demo_led_io.pin        = FL_GPIO_PIN_0 | FL_GPIO_PIN_1 | FL_GPIO_PIN_5 
                           | FL_GPIO_PIN_6 | FL_GPIO_PIN_8 | FL_GPIO_PIN_12 
                           | FL_GPIO_PIN_13 | FL_GPIO_PIN_14;
    FL_GPIO_Init(GPIOD, &demo_led_io);
    
    DIG1_PWR_OFF();
    DIG2_PWR_OFF();
    SEG_A_OFF();
    SEG_B_OFF();
    SEG_C_OFF();
    SEG_D_OFF();
    SEG_E_OFF();
    SEG_F_OFF();
    SEG_G_OFF();
    SEG_DP_OFF();
    
    /* Timebase Timer 10ms */
    demo_btim.prescaler = FL_CMU_BSTIM16_CLK_SOURCE_APBCLK;
    demo_btim.prescaler = 24000 - 1;
    demo_btim.autoReload = 10 - 1;
    demo_btim.autoReloadState = FL_DISABLE;
    FL_BSTIM16_Init(BSTIM16, &demo_btim);
    FL_BSTIM16_ClearFlag_Update(BSTIM16);
    FL_BSTIM16_EnableIT_Update(BSTIM16);
    NVIC_EnableIRQ(BSTIM_IRQn);
    /* USER CODES AREA 2 END */

    /* Init TSI */
    TSI_Init();

    /* Enable all widgets */
    TSI_Widget_EnableAll();

    /* Feed watchdog */
    FL_IWDT_ReloadCounter(IWDT);

    /* Start TSI */
    TSI_Start();

    /* Feed watchdog */
    FL_IWDT_ReloadCounter(IWDT);
    
    /* Init and start TSI debug */
    TSI_Debug_Init();
    TSI_Debug_Start();

    /* USER CODES AREA 3 BEGIN */
    FL_BSTIM16_Enable(BSTIM16);
    /* USER CODES AREA 3 END */

    while(1)
    {   
        /* USER CODES AREA 4 BEGIN */

        /* USER CODES AREA 4 END */

        /* Feed watchdog */
        FL_IWDT_ReloadCounter(IWDT);
        
        /* Handle debug event */
        TSI_Debug_Handler();

        /* USER CODES AREA 5 BEGIN */
        
        /* USER CODES AREA 5 END */
              
        /* Wait until a scan sequence has completed */
        if(TSI_GETSTAT_SCAN_CPLT())
        {   
            TSI_CLRSTAT_SCAN_CPLT();
            
            /* Update all widgets */
            TSI_Widget_UpdateAll();
            
            /* Update debug interface trace buffer */
            TSI_Debug_UpdateTrace();

            /* USER CODES AREA 6 BEGIN */
            if(TSI_WidgetList.btn1.buttonStatus)
            {
                btnPressed = 1;
            }
            else if(TSI_WidgetList.btn2.buttonStatus)
            {
                btnPressed = 2;
            }
            else if(TSI_WidgetList.btn3.buttonStatus)
            {
                btnPressed = 3;
            }
            else if(TSI_WidgetList.btn4.buttonStatus)
            {
                btnPressed = 4;
            }
            else if(TSI_WidgetList.btn5.buttonStatus)
            {
                btnPressed = 5;
            }
            else if(TSI_WidgetList.btn6.buttonStatus)
            {
                btnPressed = 6;
            }
            else if(TSI_WidgetList.btn7.buttonStatus)
            {
                btnPressed = 7;
            }
            else if(TSI_WidgetList.btn8.buttonStatus)
            {
                btnPressed = 8;
            }
            else
            {
                /* Do nothing */
            }
            
            if(dispSliderVal == 0)
            {
                DigitLed_ShowBtnNum(btnPressed);
            }
            
            if(TSI_WidgetList.slider.base.status)
            {
                uint8_t center = TSI_WidgetList.slider.centerPos;
                uint8_t val = (uint8_t)(((uint32_t)center * 100) >> 8);
                
                H = ((uint32_t)center * 360 + 180) >> 8;

                if(V == 0)
                {
                    V = 1;
                }
                
                if(S == 0)
                {
                    S = 1;
                }
                
                /* Show slider value for 1.5s */
                dispSliderVal = 1;
                dispSliderValCnt = 150;     /* 10ms * 150 = 1.5s */
                DigitLed_ShowSliderNum(val);
            }
            DemoLed_ShowHSV(H, S, V);
            
            /* USER CODES AREA 6 END */
        }

        /* USER CODES AREA 7 BEGIN */

        /* USER CODES AREA 7 END */
    }
}

/* Private functions --------------------------------------------------------*/
/** 
 * @brief Init system clock.
 * 
 */
void SystemClockInit(void)
{
    /* USER PRE SYSTEM CLOCK INIT BEGIN */

    /* USER PRE SYSTEM CLOCK INIT END */

    /* Enable RCHF 8MHz */
    FL_CMU_RCHF_WriteTrimValue(RCHF8M_TRIM);
    FL_CMU_RCHF_SetFrequency(FL_CMU_RCHF_FREQUENCY_8MHZ);

    /* Config PLL */
    FL_CMU_PLL_Disable();
    FL_CMU_PLL_SetClockSource(FL_CMU_PLL_CLK_SOURCE_RCHF);
    FL_CMU_PLL_SetPrescaler(FL_CMU_PLL_PSC_DIV8);
    FL_CMU_PLL_WriteMultiplier(48 - 1);
    FL_CMU_PLL_SetOutputMultiplier(FL_CMU_PLL_OUTPUT_X1);
    FL_CMU_PLL_Enable();
    
    /* Wait for the PLL lock flag */
    uint32_t timeout = 0xFFFFFFFFUL;
    do
    {
        if(FL_CMU_IsActiveFlag_PLLReady() == 0x1U) 
        {
            break;
        }

        /* Clear watchdog */
        FL_IWDT_ReloadCounter(IWDT);

    } while (--timeout > 0);

    /* Set flash read wait */
    FL_FLASH_SetCodeReadWait(FLASH, FL_FLASH_READ_CODEWAIT_1CYCLE);

    /* Set system clock source and bus prescaler */
    FL_CMU_SetAHBPrescaler(FL_CMU_AHBCLK_PSC_DIV2);
    FL_CMU_SetAPBPrescaler(FL_CMU_APBCLK_PSC_DIV1);
    FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_PLL);

    /* Update system core clock */
    SystemCoreClock = 24000000;

    /* USER SYSTEM CLOCK INIT BEGIN */

    /* USER SYSTEM CLOCK INIT END */
}

/* USER PRIVATE FUNCTION BEGIN */
void DemoLed_ShowHSV(float h, float s, float v)
{
    float a,b,c;
    uint8_t R,G,B;
    uint8_t i;
    float f;

    if(s == 0)
    {
        R=G=B=v;
    }
    else
    {
        h /= 60;
        i = (uint8_t)h;
        f = h - i;
        a = v * (1-s);
        b = v * (1-s*f);
        c = v * (1-s*(1-f));
        switch(i)
        {
            case 0:
                R=v*255; G=c*255; B=a*255;
                break;
            case 1:
                R=b*255; G=v*255; B=a*255;
                break;
            case 2:
                R=a*255; G=v*255; B=c*255;
                break;
            case 3:
                R=a*255; G=b*255; B=v*255;
                break;
            case 4:
                R=c*255; G=a*255; B=v*255;
                break;
            case 5:
                R=v*255; G=a*255; B=b*255;
                break;
        }
    }

    // 调节Demo LED
    FL_TAU_WriteCompare(TAU02, G * 100 / 255);
    FL_TAU_WriteCompare(TAU13, B * 100 / 255);
    FL_TAU_WriteCompare(TAU11, R * 100 / 255);    
}

void DigitLed_ShowDigit(uint8_t digit)
{
    uint8_t bitmask = digiDispTable[digit];
    
    (bitmask & 0x1) ? SEG_A_ON() : SEG_A_OFF(); bitmask >>= 1;
    (bitmask & 0x1) ? SEG_B_ON() : SEG_B_OFF(); bitmask >>= 1;
    (bitmask & 0x1) ? SEG_C_ON() : SEG_C_OFF(); bitmask >>= 1;
    (bitmask & 0x1) ? SEG_D_ON() : SEG_D_OFF(); bitmask >>= 1;
    (bitmask & 0x1) ? SEG_E_ON() : SEG_E_OFF(); bitmask >>= 1;
    (bitmask & 0x1) ? SEG_F_ON() : SEG_F_OFF(); bitmask >>= 1;
    (bitmask & 0x1) ? SEG_G_ON() : SEG_G_OFF(); bitmask >>= 1;
    (bitmask & 0x1) ? SEG_DP_ON() : SEG_DP_OFF(); bitmask >>= 1;
}

/* 滑条数值显示 2位数值，例如：04、85 */
void DigitLed_ShowSliderNum(uint8_t num)
{
    digits[1] = num / 10;
    digits[0] = num - digits[1] * 10;
}

/* 为和滑条显示区分开来，按键显示 '-'+按键号，例如：-5 */
void DigitLed_ShowBtnNum(uint8_t num)
{
    digits[0] = num % 10;
    digits[1] = 10;
}

void BSTIM_IRQHandler(void)
{
    if(FL_BSTIM16_IsActiveFlag_Update(BSTIM16))
    {
        FL_BSTIM16_ClearFlag_Update(BSTIM16);
        
        if(currDigit == 0)
        {
            DIG1_PWR_ON();
            DIG2_PWR_OFF();
            DigitLed_ShowDigit(digits[0]);
            currDigit = 1;
        }
        else
        {
            DIG1_PWR_OFF();
            DIG2_PWR_ON();
            DigitLed_ShowDigit(digits[1]);
            currDigit = 0;
        }
        
        if(dispSliderValCnt > 0)
            dispSliderValCnt--;
        else
            dispSliderVal = 0;
    }
}
/* USER PRIVATE FUNCTION END */

